Module asymmetric

Module asymmetric 

Modules

rsa_basic
rsa.rs may be too big because of documentation and plenty of examples. So, in order to provide documentation without docs.rs’s failing generating documentation, dummy codes were made and documentation and examples were moved to rsa_basic.rs.

Structs

Asymmetric_PRNG_Creator_AES_128
A PRNG creator that produces a Random_Generic instance using the AES_128 hash algorithm as its underlying engine.
Asymmetric_PRNG_Creator_BIG_KECCAK_1024
A PRNG creator that produces a Random_Generic instance using the BIG_KECCAK_1024 hash algorithm as its underlying engine.
Asymmetric_PRNG_Creator_CPRNG_Engine
A PRNG creator that produces a Random_Generic instance using the CPRNG_Engine hash algorithm as its underlying engine.
Asymmetric_PRNG_Creator_DES
A PRNG creator that produces a Random_Generic instance using the DES hash algorithm as its underlying engine.
Asymmetric_PRNG_Creator_MD4
A PRNG creator that produces a Random_Generic instance using the MD4 hash algorithm as its underlying engine.
Asymmetric_PRNG_Creator_MD5
A PRNG creator that produces a Random_Generic instance using the MD5 hash algorithm as its underlying engine.
Asymmetric_PRNG_Creator_SHA0
A PRNG creator that produces a Random_Generic instance using the SHA0 hash algorithm as its underlying engine.
Asymmetric_PRNG_Creator_SHA1
A PRNG creator that produces a Random_Generic instance using the SHA1 hash algorithm as its underlying engine.
Asymmetric_PRNG_Creator_SHA2_256
A PRNG creator that produces a Random_Generic instance using the SHA2_256 hash algorithm as its underlying engine.
Asymmetric_PRNG_Creator_SHA2_512
A PRNG creator that produces a Random_Generic instance using the SHA2_512 hash algorithm as its underlying engine.
Asymmetric_PRNG_Creator_SHA3_256
A PRNG creator that produces a Random_Generic instance using the SHA3_256 hash algorithm as its underlying engine.
Asymmetric_PRNG_Creator_SHA3_512
A PRNG creator that produces a Random_Generic instance using the SHA3_512 hash algorithm as its underlying engine.
Asymmetric_PRNG_Creator_SHAKE_128
A PRNG creator that produces a Random_Generic instance using the SHAKE_128 hash algorithm as its underlying engine.
Asymmetric_PRNG_Creator_SHAKE_256
A PRNG creator that produces a Random_Generic instance using the SHAKE_256 hash algorithm as its underlying engine.
RSA_Generic
RSA (Ron Rivest, Adi Shamir, Leonard Adleman) is asymmetric-key encryption/decryption algorithm for the encryption of digital data

Traits

Hash
OAEP
This trait OAEP is based on PKCS #1 ver. 2.1. The RSA OAEP (Optimal Asymmetric Encryption Padding) format is a Feistel network-based padding scheme designed to provide “plaintext awareness,” preventing an attacker from modifying the ciphertext without being detected.
PKCS1V15
This trait PKCS1V15 is based on PKCS #1 ver. 1.5. The RSA PKCS #1 v1.5 padding format is designed to format a message before RSA encryption to ensure that the resulting block matches the length of the RSA modulus (n) and introduces randomness for security.
PRNG

Type Aliases

RSA_1024
RSA_2048
RSA_4096
RSA_1024_u8
RSA_1024_u16
RSA_1024_u32
RSA_1024_u64
RSA_1024_u128
RSA_2048_u8
RSA_2048_u16
RSA_2048_u32
RSA_2048_u64
RSA_2048_u128
RSA_4096_u8
RSA_4096_u16
RSA_4096_u32
RSA_4096_u64
RSA_4096_u128