SHA2_512_t

Type Alias SHA2_512_t 

pub type SHA2_512_t<const t: usize = 512> = SHA2_512_t_Generic<t>;
You have freedom of changing t for SHA-2-512/t.

Generic Parameters

You can create your own expanded version of SHA-2-512/t by changing the generic parameter t.

  • t : the truncating bit at which the output is constructed by truncating the concatenation of h0 through h7. t should be multiple of 8 and equal to or less than 512. If t is not a multiple of eight, t will be overestimated to make a multiple of eight. The default value of t is 512.

Aliased Type

pub struct SHA2_512_t<const t: usize = 512> { /* private fields */ }