SHAKE_512

Type Alias SHAKE_512 

pub type SHAKE_512 = Keccak_Generic<72, { KECCAK_CONST::SHAKE }>;
SHAKE_512 is non-standard hash alogrithm which is created by expanding the Keccak family members. According to the descryption about Keccak:

  • The parameter L = 6
  • The parameter W = 2 ^ L (= 6) = 64 bits which means u64
  • The parameter B = 25 X W (=64) = 1600 bits (= 200 bytes) which is the whole size of the state, [[u64; 5]; 5]
  • The parameter C = 2 X security level (= 512) = 1024 bits
  • The parameter R = B (= 1600) - C (= 1024) = 576 bits (= 72 bytes)
  • The number of Rounds is 12 + 2 X L (= 6) = 24 rounds

Aliased Type

pub struct SHAKE_512 { /* private fields */ }