Random

Type Alias Random 

pub type Random = Random_BIG_KECCAK_1024;
The type Random which is a random number generator and is a synonym of Random_BIG_KECCAK_1024 at the moment. It means Random uses a hash algorithm Random_BIG_KECCAK_1024. It is cryptographically securer than its counterpart type Any and a bit slower than Any. In the near future, Random may be silently changed to use better alogrithm. If you want to keep using BIG_KECCAK_1024 for a pseudo-random number generator, you may want to use Random_BIG_KECCAK_1024. If you are happy that you will automatically use the better algotrithm in the future, you may want to use Random.

Aliased Type

pub struct Random {}