BigUInt

Struct BigUInt 

pub struct BigUInt<T, const N: usize>
where T: TraitsBigUInt<T>,
{ /* private fields */ }
big_uint.rs was too big because of documentation and plenty of examples So, in order to provide documentation without docs.rs’s failing generating documentation, dummy codes were made and documentation and examples were moved to big_uint_arithmetic.rs.